@@ -9,8 +9,6 @@
 -- Compilation of symbolic programs to C
 -----------------------------------------------------------------------------
 
-{-# LANGUAGE CPP           #-}
-{-# LANGUAGE PatternGuards #-}
 {-# LANGUAGE TupleSections #-}
 
 {-# OPTIONS_GHC -Wall -Werror -Wno-incomplete-uni-patterns #-}
@@ -33,6 +31,7 @@ import Text.PrettyPrint.HughesPJ
 import qualified Text.PrettyPrint.HughesPJ as P ((<>))
 
 import Data.SBV.Core.Data
+import Data.SBV.Core.Kind (kRoundingMode)
 import Data.SBV.Compilers.CodeGen
 
 import Data.SBV.Utils.PrettyNum   (chex, showCFloat, showCDouble)
@@ -198,6 +197,7 @@ showCType i = case kindOf i of
 -- | The printf specifier for the type
 specifier :: CgConfig -> SV -> Doc
 specifier cfg sv = case kindOf sv of
+                     KVar{}        -> die $ "variable sort: " ++ show (kindOf sv)
                      KBool         -> spec (False, 1)
                      KBounded b i  -> spec (b, i)
                      KUnbounded    -> spec (True, fromJust (cgInteger cfg))
@@ -210,10 +210,9 @@ specifier cfg sv = case kindOf sv of
                      KFP{}         -> die   "arbitrary float sort"
                      KList k       -> die $ "list sort: "   ++ show k
                      KSet  k       -> die $ "set sort: "    ++ show k
-                     KUserSort s _ -> die $ "user sort: "   ++ s
+                     KApp s _      -> die $ "ADT app: "     ++ s
+                     KADT s _ _    -> die $ "ADT: "         ++ s
                      KTuple k      -> die $ "tuple sort: "  ++ show k
-                     KMaybe  k     -> die $ "maybe sort: "  ++ show k
-                     KEither k1 k2 -> die $ "either sort: " ++ show (k1, k2)
                      KArray  k1 k2 -> die $ "array sort: "  ++ show (k1, k2)
   where u8InHex = cgShowU8InHex cfg
 
@@ -463,10 +462,6 @@ genCProg cfg fn proto (Result pinfo kindInfo _tvals _ovals cgs topInps (_, preCo
   = notyet "Lists (SList)"
   | any isTuple kindInfo
   = notyet "Tuples (STupleN)"
-  | any isMaybe kindInfo
-  = notyet "Optional (SMaybe) values"
-  | any isEither kindInfo
-  = notyet "Either (SEither) values"
   | isNothing (cgReal cfg) && KReal `Set.member` kindInfo
   = error $ "SBV->C: SReal values are not supported by the C compiler."
           ++ "\nUse 'cgSRealType' to specify a custom type for SReal representation."
@@ -489,7 +484,7 @@ genCProg cfg fn proto (Result pinfo kindInfo _tvals _ovals cgs topInps (_, preCo
        asserts | cgIgnoreAsserts cfg = []
                | True                = origAsserts
 
-       usorts = [s | KUserSort s _ <- Set.toList kindInfo, s /= "RoundingMode"] -- No support for any sorts other than RoundingMode!
+       usorts = [s | k@(KADT s _ _) <- Set.toList kindInfo, isADT k && not (isRoundingMode k)] -- No support for any sorts other than RoundingMode!
 
        pre    =  text "/* File:" <+> doubleQuotes (nm P.<> text ".c") P.<> text ". Automatically generated by SBV. Do not edit! */"
               $$ text ""
@@ -533,7 +528,8 @@ genCProg cfg fn proto (Result pinfo kindInfo _tvals _ovals cgs topInps (_, preCo
                ResultLamInps is       -> die $ "Unexpected inputs  : " ++ show is
 
        typeWidth = getMax 0 $ [len (kindOf s) | (s, _) <- assignments] ++ [len (kindOf s) | NamedSymVar s _ <- ins]
-                where len KReal{}            = 5
+                where len (KVar s)           = die $ "Variable: " ++ s
+                      len KReal{}            = 5
                       len KFloat{}           = 6 -- SFloat
                       len KDouble{}          = 7 -- SDouble
                       len KString{}          = 7 -- SString
@@ -547,10 +543,9 @@ genCProg cfg fn proto (Result pinfo kindInfo _tvals _ovals cgs topInps (_, preCo
                       len (KList s)          = die $ "List sort: "   ++ show s
                       len (KSet  s)          = die $ "Set sort: "    ++ show s
                       len (KTuple s)         = die $ "Tuple sort: "  ++ show s
-                      len (KMaybe k)         = die $ "Maybe sort: "  ++ show k
-                      len (KEither k1 k2)    = die $ "Either sort: " ++ show (k1, k2)
                       len (KArray  k1 k2)    = die $ "Array sort:  " ++ show (k1, k2)
-                      len (KUserSort s _)    = die $ "Uninterpreted sort: " ++ s
+                      len (KApp s _)         = die $ "Uninterpreted ADT app: " ++ s
+                      len (KADT s _ _)       = die $ "Uninterpreted ADT: "     ++ s
 
                       getMax 8 _      = 8  -- 8 is the max we can get with SInteger, so don't bother looking any further
                       getMax m []     = m
@@ -695,21 +690,21 @@ handleIEEE w consts as var = cvt w
         -- grab the rounding-mode, if present, and make sure it's RoundNearestTiesToEven. Otherwise skip.
         fpArgs = case as of
                    []            -> []
-                   ((m, _):args) -> case kindOf m of
-                                      KUserSort "RoundingMode" _ -> case checkRM (m `lookup` consts) of
-                                                                      Nothing          -> args
-                                                                      Just (Left  msg) -> die msg
-                                                                      Just (Right msg) -> tbd msg
-                                      _                          -> as
+                   ((m, _):args)
+                     | isRoundingMode m -> case checkRM (m `lookup` consts) of
+                                             Nothing          -> args
+                                             Just (Left  msg) -> die msg
+                                             Just (Right msg) -> tbd msg
+                     | True              -> as
 
         -- Check that the RM is RoundNearestTiesToEven.
         -- If we start supporting other rounding-modes, this would be the point where we'd insert the rounding-mode set/reset code
         -- instead of merely returning OK or not
-        checkRM (Just cv@(CV (KUserSort "RoundingMode" _) v)) =
-              case v of
-                CUserSort (_, "RoundNearestTiesToEven") -> Nothing
-                CUserSort (_, s)                        -> Just (Right $ "handleIEEE: Unsupported rounding-mode: " ++ show s ++ " for: " ++ show w)
-                _                                       -> Just (Left  $ "handleIEEE: Unexpected value for rounding-mode: " ++ show cv ++ " for: " ++ show w)
+        checkRM (Just cv@(CV k v))
+          | k == kRoundingMode = case v of
+                                   CADT ("RoundNearestTiesToEven", []) -> Nothing
+                                   CADT (s,                        []) -> Just (Right $ "handleIEEE: Unsupported rounding-mode: " ++ show s ++ " for: " ++ show w)
+                                   _                                   -> Just (Left  $ "handleIEEE: Unexpected value for rounding-mode: " ++ show cv ++ " for: " ++ show w)
         checkRM (Just cv) = Just (Left  $ "handleIEEE: Expected rounding-mode, but got: " ++ show cv ++ " for: " ++ show w)
         checkRM Nothing   = Just (Right $ "handleIEEE: Non-constant rounding-mode for: " ++ show w)
 
@@ -800,6 +795,7 @@ ppExpr cfg consts (SBVApp op opArgs) lhs (typ, var)
                 canOverflow False sz = (2::Integer)^sz    -1 >= fromIntegral len
 
                 (needsCheckL, needsCheckR) = case k of
+                                               KVar{}          -> die $ "array index with variable: " ++ show k
                                                KBool           -> (False, canOverflow False (1::Int))
                                                KBounded sg sz  -> (sg, canOverflow sg sz)
                                                KReal           -> die "array index with real value"
@@ -815,10 +811,9 @@ ppExpr cfg consts (SBVApp op opArgs) lhs (typ, var)
                                                KList     s     -> die $ "List sort "   ++ show s
                                                KSet      s     -> die $ "Set sort "    ++ show s
                                                KTuple    s     -> die $ "Tuple sort "  ++ show s
-                                               KMaybe    ek    -> die $ "Maybe sort "  ++ show ek
-                                               KEither   k1 k2 -> die $ "Either sort " ++ show (k1, k2)
                                                KArray    k1 k2 -> die $ "Array  sort " ++ show (k1, k2)
-                                               KUserSort s _   -> die $ "Uninterpreted sort: " ++ s
+                                               KApp      s _   -> die $ "ADT app: " ++ s
+                                               KADT      s _ _ -> die $ "ADT: "     ++ s
 
         -- Div/Rem should be careful on 0, in the SBV world x `div` 0 is 0, x `rem` 0 is x
         -- NB: Quot is supposed to truncate toward 0; Not clear to me if C guarantees this behavior.
