@@ -90,7 +90,7 @@ instance HasKind a => HasKind (RCSet a) where
 -- That is, we store the history of the writes. The earlier a pair is in the list, the "later" it
 -- is done, i.e., it takes precedence over the latter entries.
 data ArrayModel a b = ArrayModel [(a, b)] b
-                     deriving (G.Data, Generic, NFData)
+                     deriving (G.Data, Generic, NFData, Show)
 
 -- | The kind of an ArrayModel
 instance (HasKind a, HasKind b) => HasKind (ArrayModel a b) where
@@ -425,7 +425,7 @@ showCV shk w = sh (cvVal w) ++ kInfo
 
 -- | Create a constant word from an integral.
 mkConstCV :: Integral a => Kind -> a -> CV
-mkConstCV k@(KVar{})      _ = error $ "mkConstCV: Unexpected kind: " ++ show k
+mkConstCV k@KVar{}        _ = error $ "mkConstCV: Unexpected kind: " ++ show k
 mkConstCV KBool           a = normCV $ CV KBool      (CInteger  (toInteger a))
 mkConstCV k@KBounded{}    a = normCV $ CV k          (CInteger  (toInteger a))
 mkConstCV KUnbounded      a = normCV $ CV KUnbounded (CInteger  (toInteger a))
