@@ -231,11 +231,11 @@ class SatModel a where
   -- | Given a sequence of constant-words, extract one instance of the type @a@, returning
   -- the remaining elements untouched. If the next element is not what's expected for this
   -- type you should return 'Nothing'
-  parseCVs  :: [CV] -> Maybe (a, [CV])
+  parseCVs :: [CV] -> Maybe (a, [CV])
 
   -- | Given a parsed model instance, transform it using @f@, and return the result.
   -- The default definition for this method should be sufficient in most use cases.
-  cvtModel  :: (a -> Maybe b) -> Maybe (a, [CV]) -> Maybe (b, [CV])
+  cvtModel :: (a -> Maybe b) -> Maybe (a, [CV]) -> Maybe (b, [CV])
   cvtModel f x = x >>= \(a, r) -> f a >>= \b -> return (b, r)
 
   {-# MINIMAL parseCVs #-}
@@ -319,6 +319,18 @@ instance (KnownNat n, BVIsNonZero n) => SatModel (WordN n) where
 instance (KnownNat n, BVIsNonZero n) => SatModel (IntN n) where
   parseCVs = genParse (kindOf (undefined :: IntN n))
 
+-- | Constructing models for t'ArrayModel'
+instance (SatModel k, SatModel v) => SatModel (ArrayModel k v) where
+  parseCVs (CV (KArray kk kv) (CArray (ArrayModel tbl def)) : r)
+    | Just (def', _) <- parseCVs @v [CV kv def]
+    , let convert (k, v) = do
+            (k', _) <- parseCVs @k [CV kk k]
+            (v', _) <- parseCVs @v [CV kv v]
+            pure (k', v')
+    , Just tbl' <- traverse convert tbl
+    = Just (ArrayModel tbl' def', r)
+  parseCVs _ = Nothing
+
 -- | @CV@ as extracted from a model; trivial definition
 instance SatModel CV where
   parseCVs (cv : r) = Just (cv, r)
