@@ -25,6 +25,7 @@ module Data.SBV.TP.Kernel (
        , inductiveLemma, inductiveLemmaWith
        , internalAxiom
        , TPProofContext (..), smtProofStep, HasInductionSchema(..)
+       , tpMergeCfg
        ) where
 
 import Control.Monad.Trans  (liftIO, MonadIO)
@@ -186,33 +187,40 @@ internalAxiom nm p = Proof $ ProofObj { dependencies = []
                                       , isCached     = False
                                       }
 
--- | Prove a lemma, using the given configuration
-lemmaWith :: Proposition a => SMTConfig -> String -> a -> [ProofObj] -> TP (Proof a)
-lemmaWith cfg@SMTConfig{tpOptions = TPOptions{printStats}} nm inputProp by = withProofCache nm $ do
-                 tpSt <- getTPState
-                 u    <- tpGetNextUnique
-                 liftIO $ getTimeStampIf printStats >>= runSMTWith cfg . go tpSt u
-  where go tpSt u mbStartTime = do qSaturateSavingObservables inputProp
-                                   mapM_ (constrain . getObjProof) by
-                                   query $ smtProofStep cfg tpSt "Lemma" 0 (TPProofOneShot nm by) Nothing inputProp [] (good mbStartTime u)
-
-        -- What to do if all goes well
-        good mbStart u d = do mbElapsed <- getElapsedTime mbStart
-                              liftIO $ finishTP cfg ("Q.E.D." ++ concludeModulo by) d $ catMaybes [mbElapsed]
-                              pure $ Proof $ ProofObj { dependencies = by
-                                                      , isUserAxiom  = False
-                                                      , getObjProof  = label nm (quantifiedBool inputProp)
-                                                      , getProp      = toDyn inputProp
-                                                      , proofName    = nm
-                                                      , uniqId       = u
-                                                      , isCached     = False
-                                                      }
+-- | Propagate the settings for ribbon/timing from top to current. Because in any subsequent configuration
+-- in a lemmaWith, inductWith etc., we just want to change the solver, not the actual settings for TP.
+tpMergeCfg :: SMTConfig -> SMTConfig -> SMTConfig
+tpMergeCfg cur top = cur{tpOptions = tpOptions top}
 
 -- | Prove a given statement, using auxiliaries as helpers. Using the default solver.
 lemma :: Proposition a => String -> a -> [ProofObj] -> TP (Proof a)
 lemma nm f by = do cfg <- getTPConfig
                    lemmaWith cfg nm f by
 
+-- | Prove a lemma, using the given configuration.
+lemmaWith :: Proposition a => SMTConfig -> String -> a -> [ProofObj] -> TP (Proof a)
+lemmaWith cfgIn nm inputProp by = withProofCache nm $ do
+                 topCfg <- getTPConfig
+                 let cfg@SMTConfig{tpOptions = TPOptions{printStats}} = cfgIn `tpMergeCfg` topCfg
+                 tpSt <- getTPState
+                 u    <- tpGetNextUnique
+                 liftIO $ getTimeStampIf printStats >>= runSMTWith cfg . go tpSt cfg u
+  where go tpSt cfg u mbStartTime = do qSaturateSavingObservables inputProp
+                                       mapM_ (constrain . getObjProof) by
+                                       query $ smtProofStep cfg tpSt "Lemma" 0 (TPProofOneShot nm by) Nothing inputProp [] (good cfg mbStartTime u)
+
+        -- What to do if all goes well
+        good cfg mbStart u d = do mbElapsed <- getElapsedTime mbStart
+                                  liftIO $ finishTP cfg ("Q.E.D." ++ concludeModulo by) d $ catMaybes [mbElapsed]
+                                  pure $ Proof $ ProofObj { dependencies = by
+                                                          , isUserAxiom  = False
+                                                          , getObjProof  = label nm (quantifiedBool inputProp)
+                                                          , getProp      = toDyn inputProp
+                                                          , proofName    = nm
+                                                          , uniqId       = u
+                                                          , isCached     = False
+                                                          }
+
 -- | Prove a given statement, using the induction schema for the proposition. Using the default solver.
 inductiveLemma :: Inductive a => String -> a -> [ProofObj] -> TP (Proof a)
 inductiveLemma nm f by = do cfg <- getTPConfig
