@@ -34,7 +34,7 @@ module Data.SBV.TP.TP (
        , (|-), (|->), (⊢), (=:), (≡), (??), (∵), split, split2, cases, (==>), (⟹), qed, trivial, contradiction
        , qc, qcWith
        , disp
-       , recall
+       , recall, recallWith
        ) where
 
 import Data.SBV
@@ -91,11 +91,6 @@ proofTreeSaturatables = go
 getCalcStrategySaturatables :: CalcStrategy -> [SBool]
 getCalcStrategySaturatables (CalcStrategy calcIntros calcProofTree _calcQCInstance) = calcIntros : proofTreeSaturatables calcProofTree
 
--- | Propagate the settings for ribbon/timing from top to current. Because in any subsequent configuration
--- in a lemmaWith, inductWith etc., we just want to change the solver, not the actual settings for TP.
-tpMergeCfg :: SMTConfig -> SMTConfig -> SMTConfig
-tpMergeCfg cur top = cur{tpOptions = tpOptions top}
-
 -- | Use an injective type family to allow for curried use of calc and strong induction steps.
 type family StepArgs a t = result | result -> t where
   StepArgs                                                                             SBool  t =                                               (SBool, TPProofRaw (SBV t))
@@ -1525,15 +1520,26 @@ infix 0 ⟹
 -- | Recalling a proof. This essentially sets the verbose output off during this proof. Note that
 -- if we're doing stats, we ignore this as the whole point of doing stats is to see steps in detail.
 recall :: String -> TP (Proof a) -> TP (Proof a)
-recall nm prf = do
-  cfg <- getTPConfig
-  if printStats (tpOptions cfg)
-     then prf
-     else do tab <- liftIO $ startTP cfg (verbose cfg) "Lemma" 0 (TPProofOneShot nm [])
-             setTPConfig cfg{tpOptions = (tpOptions cfg) {quiet = True}}
-             r@Proof{proofOf = ProofObj{dependencies}} <- prf
-             setTPConfig cfg
-             liftIO $ finishTP cfg ("Q.E.D." ++ concludeModulo dependencies) (tab, Nothing) []
-             pure r
+recall nm prf = getTPConfig >>= \cfg -> recallWith cfg nm prf
 
-{- HLint ignore module "Eta reduce" -}
+-- | Recalling a proof, using a given config. We keep the stat field as the or of the current and the context
+-- configuration.
+recallWith :: SMTConfig -> String -> TP (Proof a) -> TP (Proof a)
+recallWith cfgIn nm prf = do
+  topCfg <- getTPConfig
+  let cfg@SMTConfig{tpOptions = TPOptions{printStats}} = cfgIn `tpMergeCfg` topCfg
+  if printStats
+     then do restoring cfg topCfg prf
+     else do tab <- liftIO $ startTP cfg (verbose cfg) "Lemma" 0 (TPProofOneShot nm [])
+             let new = cfg{tpOptions = (tpOptions cfg) {quiet = True}}
+             restoring new topCfg $ do
+                 r@Proof{proofOf = ProofObj{dependencies}} <- prf
+                 liftIO $ finishTP cfg ("Q.E.D." ++ concludeModulo dependencies) (tab, Nothing) []
+                 pure r
+ where restoring new old act = do setTPConfig new
+                                  res <- act
+                                  setTPConfig old
+                                  pure res
+
+{- HLint ignore module "Eta reduce"         -}
+{- HLint ignore module "Reduce duplication" -}
