@@ -35,11 +35,9 @@
 -- NB. Thanks to Amit Goel for suggesting the formalization strategy used in here.
 -----------------------------------------------------------------------------
 
-{-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
-{-# LANGUAGE FlexibleInstances   #-}
-{-# LANGUAGE StandaloneDeriving  #-}
-{-# LANGUAGE TemplateHaskell     #-}
+{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE TemplateHaskell   #-}
+{-# LANGUAGE TypeApplications  #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -53,14 +51,12 @@ import Data.SBV
 
 -- | Months. We only put in the months involved in the puzzle for simplicity
 data Month = May | Jun | Jul | Aug
-           deriving (Enum, Bounded)
 
 -- | Days. Again, only the ones mentioned in the puzzle.
 data Day = D14 | D15 | D16 | D17 | D18 | D19
-         deriving (Enum, Bounded)
 
-mkSymbolicEnumeration ''Month
-mkSymbolicEnumeration ''Day
+mkSymbolic [''Month]
+mkSymbolic [''Day]
 
 -- | Represent the birthday as a record
 data Birthday = BD SMonth SDay
