@@ -31,6 +31,7 @@ import Data.SBV.Tuple
 
 #ifdef DOCTEST
 -- $setup
+-- >>> import Data.SBV
 -- >>> import Data.SBV.TP
 #endif
 
@@ -237,7 +238,9 @@ dvdAbs = do
 -- Lemma: dvdMul
 --   Step: 1 (2 way case split)
 --     Step: 1.1                           Q.E.D.
---     Step: 1.2                           Q.E.D.
+--     Step: 1.2.1                         Q.E.D.
+--     Step: 1.2.2                         Q.E.D.
+--     Step: 1.2.3                         Q.E.D.
 --     Step: 1.Completeness                Q.E.D.
 --   Result:                               Q.E.D.
 -- [Proven] dvdMul :: Ɐd ∷ Integer → Ɐa ∷ Integer → Ɐk ∷ Integer → Bool
@@ -251,7 +254,9 @@ dvdMul = calc "dvdMul"
                                          =: qed
                               , d ./= 0 ==> d `dvd` (k*a)
                                          ?? a .== d * a `sEDiv` d
-                                         =: d `dvd` (k * d * a `sEDiv` d)
+                                         =: d `dvd` ((d * a `sEDiv` d) * k)
+                                         =: d `dvd` (d * ((a `sEDiv` d) * k))
+                                         =: sTrue
                                          =: qed
                               ]
 
@@ -308,7 +313,7 @@ dvdEvenWhenOdd = calc "dvdEvenWhenOdd"
                             -- Arithmetic gives us
                             =: 2*a .== 2*t*m + m .&& 2*(a-t*m) .== m
 
-                            -- So, we now now m is even
+                            -- So, we now know m is even
                             =: 2 `sDivides` m
 
                             -- Give that divisor a name:
@@ -676,8 +681,14 @@ gcdAdd = do
 -- | \(\gcd\, (2a)\, (2b) = 2 (\gcd\,a\, b)\)
 --
 -- ==== __Proof__
--- >>> runTP gcdEvenEven
--- Lemma: modEE                            Q.E.D.
+-- >>> runTPWith cvc5 gcdEvenEven
+-- Lemma: red2                             Q.E.D.
+-- Lemma: modEE
+--   Step: 1                               Q.E.D.
+--   Step: 2                               Q.E.D.
+--   Step: 3                               Q.E.D.
+--   Step: 4                               Q.E.D.
+--   Result:                               Q.E.D.
 -- Inductive lemma (strong): nGCDEvenEven
 --   Step: Measure is non-negative         Q.E.D.
 --   Step: 1 (2 way case split)
@@ -698,10 +709,21 @@ gcdAdd = do
 gcdEvenEven :: TP (Proof (Forall "a" Integer -> Forall "b" Integer -> SBool))
 gcdEvenEven = do
 
-   modEE <- lemma "modEE"
-                  (\(Forall @"a" a) (Forall @"b" b) -> b ./= 0 .=> (2 * a) `sEMod` (2 * b) .== 2 * (a `sEMod` b))
+   red2  <- lemma "red2"
+                  (\(Forall @"a" a) (Forall @"b" b) -> b ./= 0 .=> (2*a) `sEDiv` (2*b) .== a `sEDiv` b)
                   []
 
+   modEE <- calc "modEE"
+                 (\(Forall @"a" a) (Forall @"b" b) -> b ./= 0 .=> (2*a) `sEMod` (2*b) .== 2 * (a `sEMod` b)) $
+                 \a b -> [b ./= 0]
+                      |- (2*a) `sEMod` (2*b)
+                      =: 2*a - 2*b * ((2*a) `sEDiv` (2*b))
+                      ?? red2 `at` (Inst @"a" a, Inst @"b" b)
+                      =: 2*a - 2*b * (a `sEDiv` b)
+                      =: 2 * (a - b * (a `sEDiv` b))
+                      =: 2 * (a `sEMod` b)
+                      =: qed
+
    nGCDEvenEven <- sInduct "nGCDEvenEven"
                            (\(Forall @"a" a) (Forall @"b" b) -> a .>= 0 .&& b .>= 0 .=> nGCD (2*a) (2*b) .== 2 * nGCD a b)
                            (\_a b -> b, []) $
@@ -944,10 +966,10 @@ gcdBin a b = nGCDBin (abs a) (abs b)
 -- [Proven] gcdBinEquiv :: Ɐa ∷ Integer → Ɐb ∷ Integer → Bool
 gcdBinEquiv :: TP (Proof (Forall "a" Integer -> Forall "b" Integer -> SBool))
 gcdBinEquiv = do
-   gEvenEven <- recall "gcdEvenEven" gcdEvenEven
-   gOddEven  <- recall "gcdOddEven"  gcdOddEven
-   gAdd      <- recall "gcdAdd"      gcdAdd
-   comm      <- recall "commutative" commutative
+   gEvenEven <- recallWith cvc5 "gcdEvenEven" gcdEvenEven
+   gOddEven  <- recall          "gcdOddEven"  gcdOddEven
+   gAdd      <- recall          "gcdAdd"      gcdAdd
+   comm      <- recall          "commutative" commutative
 
    -- First prove over the non-negative numbers:
    nEq <- sInduct "nGCDBinEquiv"
