@@ -72,7 +72,7 @@ dividesProduct = calc "dividesProduct"
 --     Step: 1.1                           Q.E.D.
 --     Step: 1.2.1                         Q.E.D.
 --     Step: 1.2.2                         Q.E.D.
---     Step: 1.2.3 (hard)                  Q.E.D.
+--     Step: 1.2.3                         Q.E.D.
 --     Step: 1.2.4                         Q.E.D.
 --     Step: 1.Completeness                Q.E.D.
 --   Result:                               Q.E.D.
@@ -90,8 +90,8 @@ dividesTransitive = do
                              ?? z .== z `sEDiv` y * y
                              =: x `dvd` (z `sEDiv` y * y)
                              ?? y .== y `sEDiv` x * x
+                             ?? x `dvd` y
                              =: x `dvd` ((z `sEDiv` y) * (y `sEDiv` x * x))
-                             ?? "hard"
                              =: x `dvd` (x * ((z `sEDiv` y) * (y `sEDiv` x)))
                              ?? dp `at` (Inst @"x" x, Inst @"y" x, Inst @"z" ((z `sEDiv` y) * (y `sEDiv` x)))
                              =: sTrue
@@ -131,7 +131,7 @@ leastDivisorDivides =
                                                 =: qed
                            , n `sEMod` k ./= 0 ==> d `dvd` n .&& k .<= d .&& d .<= n
                                                 ?? d .== ld (k+1) n
-                                                ?? ih
+                                                ?? ih `at` (Inst @"k" (k+1), Inst @"n" n)
                                                 =: sTrue
                                                 =: qed
                            ]
@@ -172,7 +172,9 @@ leastDivisorIsLeast =
 -- Lemma: leastDivisorIsLeast              Q.E.D.
 -- Lemma: helper1                          Q.E.D.
 -- Lemma: helper2                          Q.E.D.
--- Lemma: helper3                          Q.E.D.
+-- Lemma: helper3
+--   Step: 1                               Q.E.D.
+--   Result:                               Q.E.D.
 -- Lemma: helper4                          Q.E.D.
 -- Lemma: helper5
 --   Step: 1                               Q.E.D.
@@ -185,7 +187,8 @@ leastDivisorTwice = do
   ldd <- recall "leastDivisorDivides" leastDivisorDivides
   ldl <- recall "leastDivisorIsLeast" leastDivisorIsLeast
 
-  h1 <- lemma "helper1"
+  h1 <- lemmaWith cvc5
+              "helper1"
               (\(Forall @"k" k) (Forall @"n" n) -> n .>= k .&& k .>= 2 .=> ld k (ld k n) `dvd` ld k n .&& ld k (ld k n) .<= ld k n)
               [proofOf ldd]
 
@@ -193,9 +196,15 @@ leastDivisorTwice = do
               (\(Forall @"k" k) (Forall @"n" n) -> n .>= k .&& k .>= 2 .=> ld k n `dvd` n)
               [proofOf ldd]
 
-  h3 <- lemma "helper3"
-              (\(Forall @"k" k) (Forall @"n" n) -> n .>= k .&& k .>= 2 .=> ld k (ld k n) `dvd` n)
-              [proofOf h1, proofOf h2, proofOf dt]
+  h3 <- calc "helper3"
+             (\(Forall @"k" k) (Forall @"n" n) -> n .>= k .&& k .>= 2 .=> ld k (ld k n) `dvd` n) $
+             \k n -> [n .>= k, k .>= 2]
+                  |- ld k (ld k n) `dvd` n
+                  ?? h1
+                  ?? h2
+                  ?? dt `at` (Inst @"x" (ld k (ld k n)), Inst @"y" (ld k n), Inst @"z" n)
+                  =: sTrue
+                  =: qed
 
   h4 <- lemma "helper4"
               (\(Forall @"k" k) (Forall @"n" n) -> n .>= k .&& k .>= 2 .=> k .<= ld k (ld k n))
@@ -242,7 +251,7 @@ primeAtLeast2 = lemma "primeAtLeast2" (\(Forall p) -> isPrime p .=> p .>= 2) []
 -- [Proven] leastDivisorIsPrime :: Ɐn ∷ Integer → Bool
 leastDivisorIsPrime :: TP (Proof (Forall "n" Integer -> SBool))
 leastDivisorIsPrime = do
-   ldt <- recall "leastDivisorTwice" leastDivisorTwice
+   ldt <- recall "leastDivisorTwice"   leastDivisorTwice
    ldd <- recall "leastDivisorDivides" leastDivisorDivides
 
    calc "leastDivisorIsPrime"
