@@ -10,11 +10,9 @@
 -- the constant folding based arithmetic implementation in SBV
 -----------------------------------------------------------------------------
 
-{-# LANGUAGE CPP              #-}
-{-# LANGUAGE DataKinds        #-}
-{-# LANGUAGE FlexibleContexts #-}
-{-# LANGUAGE Rank2Types       #-}
-{-# LANGUAGE TupleSections    #-}
+{-# LANGUAGE CPP           #-}
+{-# LANGUAGE Rank2Types    #-}
+{-# LANGUAGE TupleSections #-}
 
 #if MIN_VERSION_base(4,19,0)
 {-# OPTIONS_GHC -Wall -Werror -Wno-incomplete-uni-patterns -Wno-x-partial #-}
@@ -29,9 +27,7 @@ import Utils.SBVTestFramework
 
 import Data.Maybe (fromJust, fromMaybe)
 
-import qualified Data.Char     as C
-
-import qualified Data.SBV.Char () -- instances only
+import qualified Data.Char as C
 
 -- Test suite
 tests :: TestTree
@@ -162,8 +158,6 @@ genQRems = map mkTest $
         -- Haskell's divMod and quotRem differs from SBV's in two ways:
         --     - when y is 0, Haskell throws an exception, SBV sets the result to 0; like in division
         --     - Haskell overflows if x == minBound and y == -1 for bounded signed types; but SBV returns minBound, 0; which is more meaningful
-        -- NB. There was a ticket filed against the second anomaly above, See: http://ghc.haskell.org/trac/ghc/ticket/8695
-        -- But the Haskell folks decided not to fix it. Sigh..
         overflow x y = x == minBound && y == -1
         divMod0  x y = if y == 0       then (0, x) else x `divMod`   y
         divMod1  x y = if overflow x y then (x, 0) else x `divMod0`  y
