@@ -9,12 +9,8 @@
 -- Test tuples.
 -----------------------------------------------------------------------------
 
-{-# LANGUAGE DataKinds           #-}
-{-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
 {-# LANGUAGE FlexibleInstances   #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE StandaloneDeriving  #-}
 {-# LANGUAGE TemplateHaskell     #-}
 {-# LANGUAGE TypeApplications    #-}
 
@@ -32,11 +28,8 @@ import qualified Data.SBV.List as L
 
 import Utils.SBVTestFramework
 
-data E = A | B | C deriving (Bounded, Enum)
-mkSymbolicEnumeration ''E
-
-__unused :: SE
-__unused = error "stop GHC from complaining unused names" sA sB sC
+data E = A | B | C deriving Show
+mkSymbolic [''E]
 
 -- Test suite
 tests :: TestTree
