@@ -9,12 +9,10 @@
 -- Test suite for Documentation.SBV.Examples.Uninterpreted.AUF
 -----------------------------------------------------------------------------
 
-{-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
 {-# LANGUAGE FlexibleInstances   #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE StandaloneDeriving  #-}
 {-# LANGUAGE TemplateHaskell     #-}
+{-# LANGUAGE TypeApplications    #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -24,8 +22,8 @@ import Data.SBV.Control
 
 import Utils.SBVTestFramework
 
-data BinOp  = Plus | Minus | Times deriving (Enum, Bounded, Ord, Eq)
-mkSymbolicEnumeration ''BinOp
+data BinOp  = Plus | Minus | Times
+mkSymbolic [''BinOp]
 
 -- Test suite
 tests :: TestTree
