@@ -9,12 +9,10 @@
 -- Testing uninterpreted value extraction
 -----------------------------------------------------------------------------
 
-{-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
 {-# LANGUAGE FlexibleInstances   #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE StandaloneDeriving  #-}
 {-# LANGUAGE TemplateHaskell     #-}
+{-# LANGUAGE TypeApplications    #-}
 
 {-# OPTIONS_GHC -Wall -Werror #-}
 
@@ -23,8 +21,8 @@ module TestSuite.Queries.Uninterpreted where
 import Data.SBV.Control
 import Utils.SBVTestFramework
 
-data L = A | B deriving (Enum, Bounded)
-mkSymbolicEnumeration ''L
+data L = A | B deriving Show
+mkSymbolic [''L]
 
 -- Test suite
 tests :: TestTree
